<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00004006DocumentoContratoFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function limparAnexoDoContrato(array $params) {
        $sql = "UPDATE FCT_CONTRATODOC
                   SET NMARQUIVO    = NULL,
                       DSCAMINHOARQ = NULL,
                       DSTIPO       = NULL,
                       VRTAMANHO    = NULL
                 WHERE NRSEQCONTDOC = :NRSEQCONTDOC
                   AND NRORG        = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getDocumentoContrato(array $params) {
        $sql = "SELECT NRSEQCONTDOC,
                       TXMODELODOCORCA,
                       NMARQUIVO,
                       DSCAMINHOARQ,
                       DSTIPO,
                       VRTAMANHO
                  FROM FCT_CONTRATODOC
                 WHERE NRSEQCONTDOC = :NRSEQCONTDOC
                   AND NRORG        = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function atualizarCaminhoDocumentoContrato(array $params) {
        $sql = "UPDATE FCT_CONTRATODOC
                   SET DSCAMINHOARQ = :DSCAMINHOARQ,
                       DTULTATU     = SYSDATE,
                       CDOPERULTATU = :CDOPERULTATU,
                       NRORGULTATU  = :NRORGULTATU
                 WHERE NRSEQCONTDOC = :NRSEQCONTDOC
                   AND NRORG        = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Substitui os dados de alteração contratual
     */
    public function getDadosAlteracaoContratual(array $params) {
        $sql = "SELECT DSALTERACONTRATUAIS
                  FROM FCT_CONTRATO
                 WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                   AND NRORG = :NRORG";
               
       $ps = $this->connection->prepare($sql);
       $ps->execute($params);
       return $ps->fetchAll();
    }
    
    /**
     * Retorna os dados de assinatura da proposta
     */
    public function getAssinaturasContrato(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ASSINAT.*
                  FROM (SELECT
                            FCT_ASSINATDIGITALCONTR.NRASSDIGITALCONTR,
                            FCT_ASSINATDIGITALCONTR.NRSEQCONTRATO,
                            FCT_ASSINATDIGITALCONTR.NRPARCNEGOCIO,
                            FCT_ASSINATDIGITALCONTR.NRSEQINTERLOCUTOR,
                            DECODE(FCT_ASSINATDIGITALCONTR.NRPARCNEGOCIO, NULL, CRM_INTERLOCUTOR.NMINTERLOCUTOR, PARCNEGOCIO.NMPRINCIPALPARC) AS NMSIGNATARIO,
                            FCT_ASSINATDIGITALCONTR.NRCPFSIGNATARIO,
                            FCT_ASSINATDIGITALCONTR.IDTIPOASSINATURA,
                            FCT_ASSINATDIGITALCONTR.IDTESTEMUNHA,
                            FCT_ASSINATDIGITALCONTR.NRORDENACAO,
                            FCT_ASSINATDIGITALCONTR.IDSITUACAO,
                            FCT_ASSINATDIGITALCONTR.IDATIVO,
                            FCT_ASSINATDIGITALCONTR.NRORG
                        FROM
                            FCT_ASSINATDIGITALCONTR
                            LEFT JOIN PARCNEGOCIO ON FCT_ASSINATDIGITALCONTR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                                  AND FCT_ASSINATDIGITALCONTR.NRORG = PARCNEGOCIO.NRORG
                            LEFT JOIN CRM_INTERLOCUTOR ON FCT_ASSINATDIGITALCONTR.NRSEQINTERLOCUTOR = CRM_INTERLOCUTOR.NRSEQINTERLOCUTOR
                                  AND FCT_ASSINATDIGITALCONTR.NRORG = CRM_INTERLOCUTOR.NRORG
                            JOIN FCT_CONTRATO ON FCT_ASSINATDIGITALCONTR.NRSEQCONTRATO = FCT_CONTRATO.NRSEQCONTRATO
                             AND FCT_ASSINATDIGITALCONTR.NRORG = FCT_CONTRATO.NRORG
                        WHERE FCT_ASSINATDIGITALCONTR.NRSEQCONTRATO = :NRSEQCONTRATO
                          AND FCT_ASSINATDIGITALCONTR.NRORG = :NRORG
                        ORDER BY
                            FCT_ASSINATDIGITALCONTR.NRORDENACAO,
                            PARCNEGOCIO.NMPRINCIPALPARC,
                            CRM_INTERLOCUTOR.NMINTERLOCUTOR) ASSINAT";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}

?>