<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\SM\Factory\SmAssinaturaDigitalFactory as AssinaturaFactory;
//require_once "/home/developer/workfolder/sm/backend/vendor/pear/http_request2/HTTP/Request2.php";
require_once "HTTP/Request2.php";

class SmAssinaturaDigitalService {

    protected $entitymanager;
    protected $environment;
    protected $assinaturaFactory;
    protected $urlIntegracaoCertisign;

    public function __construct(EntityManager $entitymanager, Environment $environment, AssinaturaFactory $assinaturaFactory, $urlIntegracaoCertisign) {
        $this->entitymanager          = $entitymanager;
        $this->environment            = $environment;
        $this->assinaturaFactory      = $assinaturaFactory;
        $this->urlIntegracaoCertisign = $urlIntegracaoCertisign;
    }

    public function convertPDFinBase64($documento){
        try {

            $handle = fopen($documento,'r');
            $file_content = fread($handle,filesize($documento));
            fclose($handle);

            // $byte_array = unpack("C*", $file_content);

            $encoded = base64_encode($file_content);

            return $encoded;

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function buildJsontoUpload($nmDocumento, $documento){
        
        $base64 = $this->convertPDFinBase64($documento);

        $arrayupload = array(
            "fileName" => $nmDocumento,
            "bytes"  => $base64
        );

        $jsonUpload = json_encode($arrayupload, JSON_PRETTY_PRINT);

        return $jsonUpload;
    }

    public function uploadtoCertsing($nmDocumento, $documento){
        
        $token = $this->getTokenCertsign();
        
        $return = array();
        
        $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/upload', \Http_Request2::METHOD_POST);
        $url = $request->getUrl();

        $headers = array(
            // Request headers
            'Content-Type' => 'application/json',
            'Token' => $token,
        );

        $request->setHeader($headers);

        $parameters = array(
            // Request parameters
        );

        $url->setQueryVariables($parameters);

        $request->setMethod(\Http_Request2::METHOD_POST);

        $body = $this->buildJsontoUpload($nmDocumento, $documento);

        // Request body
        $request->setBody($body);

        try
        {
            $response = $request->send();
            $retorno = json_decode($response->getBody());

            $retorno = json_encode($retorno);
            
            return $retorno;
        }
        catch (\HttpException $ex)
        {
            echo $ex;

            $return['response'] = $ex;

            return $return;
        }
    }

    public function buildJsontoCreate($uploadId, $nmDocumento, $sender, $signers, $eletrSigners){
        
        /* -- Documento -- */ 
        $document = array(
            "name" => $nmDocumento,
            "upload" => array("id" => $uploadId,
                              "name" => $nmDocumento)
            );

        /* -- responsável pelo envio -- */
        $sender = array(
            "name"  => $sender['NOME'],
            "email" => $sender['EMAIL'],
            "individualIdentificationCode" => $sender['CPF']
        );
        
        /* -- Signatários Digitais -- */
        
        $signers_aux = array();
        
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }
        
        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "document" => $document,
            "sender"  => $sender,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );

        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);

        return $jsonCreate;
    }

    public function createSignatureCertsing($uploadId, $nmDocumento, $sender, $signers, $eletrSigners){
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/create', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
    
            $url->setQueryVariables($parameters);
    
            $request->setMethod(\Http_Request2::METHOD_POST);
    
            $body = $this->buildJsontoCreate($uploadId, $nmDocumento, $sender, $signers, $eletrSigners);
            $body = str_replace('\/', '/', $body);
            
            $return['request'] = $body;
            // Request body
            $request->setBody($body);
             
            try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }

        } catch(\Exception $e){
            throw $e;
        }
    }

    public function callbackCertsing($resultado) {
        try {
            
            $filedata = $resultado->getParameters();
            
            $action = $filedata['action'];
            
            if (($action == 'SIGNATURE-ELETRONIC') || ($action == 'SIGNATURE-DIGITAL')) {
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId
                );
                $getSeqPropCont = $this->assinaturaFactory->getSeqPropCont($params);
                
                if (count($getSeqPropCont) > 0) {
                    $documento   = $getSeqPropCont[0]['DOCUMENTO'];
                    $seqPropCont = $getSeqPropCont[0]['SEQUENCIAL'];
                    $tipo        = $getSeqPropCont[0]['TIPO'];
    
                    if ($tipo == 'PROPOSTA') {
                        
                        $params = array(
                            "NRSEQORCAMENTO"    => $seqPropCont,
                            "NRCPFSIGNATARIO"   => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaOrca($params);
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALORCA"];
                    
                        $params = array(
                            "IDSITUACAO"        => 'A',
                            "DSJUSTIFICARECUSA" => null,
                            "NRSEQORCADOC"      => $documento,
                            "NRASSDIGITALORCA"  => $signatario,
                            "NRSEQORCAMENTO"    => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocProposta($params);
                    }
                    else { //$tipo == 'CONTRATO'
                        $params = array(
                            "NRSEQCONTRATO"     => $seqPropCont,
                            "NRCPFSIGNATARIO"   => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaContr($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALCONTR"];
                        
                        $params = array(
                            "IDSITUACAO"        => 'A',
                            "DSJUSTIFICARECUSA" => null,
                            "NRSEQCONTDOC"      => $documento,
                            "NRASSDIGITALCONTR" => $signatario,
                            "NRSEQCONTRATO"     => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocContrato($params);
                    }
                }
            }
            else if ($action == 'SIGNATURE-NOK') {
                
                $documentId = $filedata['documentId'];
                $identifier = $filedata['identifier'];
                $message    = $filedata['message'];
                
                $params = array(
                    "IDDOCUMCERTSIGN" => $documentId
                );
                
                $getSeqPropCont = $this->assinaturaFactory->getSeqPropCont($params);
                
                if (count($getSeqPropCont) > 0) {
                    $documento   = $getSeqPropCont[0]['DOCUMENTO'];
                    $seqPropCont = $getSeqPropCont[0]['SEQUENCIAL'];
                    $tipo        = $getSeqPropCont[0]['TIPO'];
                    
                    if ($tipo == 'PROPOSTA'){
                        
                        $params = array(
                            "NRSEQORCAMENTO"    => $seqPropCont,
                            "NRCPFSIGNATARIO"   => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaOrca($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALORCA"];
                        
                        $params = array(
                            "IDSITUACAO"        => 'R',
                            "DSJUSTIFICARECUSA" => $message,
                            "NRSEQORCADOC"      => $documento,
                            "NRASSDIGITALORCA"  => $signatario,
                            "NRSEQORCAMENTO"    => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocProposta($params);
                    }
                    else{
                        
                         $params = array(
                            "NRSEQCONTRATO"     => $seqPropCont,
                            "NRCPFSIGNATARIO"   => $identifier
                        );
                        
                        $seqSignatario = $this->assinaturaFactory->recebeAssinaturaContr($params);
                        
                        if (!empty($seqSignatario))
                            $signatario = $seqSignatario[0]["NRASSDIGITALCONTR"];
                        
                        $params = array(
                            "IDSITUACAO"        => 'R',
                            "DSJUSTIFICARECUSA" => $message,
                            "NRSEQCONTDOC"      => $documento,
                            "NRASSDIGITALCONTR" => $signatario,
                            "NRSEQCONTRATO"     => $seqPropCont
                        );
                        
                        $this->assinaturaFactory->updateAssinaDocContrato($params);
                    }
                }
            }
            
            $this->verificaAceitesCallback($seqPropCont, $tipo, $signatario, $documento);
            
            //$filename = 'retono_'.$filedata['documentId'].'_'.$filedata['action'].'.txt';
            //$filedata = json_encode($filedata, JSON_PRETTY_PRINT);
            //file_put_contents('/home/developer/workfolder/retorno_certsign/'.$filename, $filedata);

        } catch(\Exception $e){
            throw $e;
        }
    }
    
    /* Verifica se todos os as situações dos documentos de assinatura para atuzalizar os registros dos signatários. Sendo:
       - Se alguns dos documentos estiver recusado,  o signatário fica como recusado.
       - Fora o caso acima, caso ainda tenha algum documento pendente, o signatário permanece pendente.
       - Caso não haja nenhum pendente e nenhum recusado, o signatário fica aprovado.
       Obs: os dados de assinatura manual serão tratados na própria ação de asssinatura manual.
    */
    public function verificaAceitesCallback($seqPropCont, $tipo, $signatario, $documento) {
        try {
            
            $countAprovado  = 0;
            $countRecusado = 0;
            $countPendente  = 0;
            $justificaRecusa = null;
            
            if ($tipo == 'PROPOSTA') {
                
                $params = array(
                    "NRASSDIGITALORCA" => $signatario,
                    "NRSEQORCAMENTO"   => $seqPropCont
                );
                
                $dadosProposta = $this->assinaturaFactory->recebeDadosAssDocProp($params);  
                
                foreach ($dadosProposta as $dados){
                    
                    if ($dados["IDSITUACAO"] == 'A'){
                        
                        $countAprovado = $countAprovado+1;
                    } 
                    else if ($dados["IDSITUACAO"] == 'R'){ 
                        
                        $countRecusado = $countRecusado+1;
                        
                        if ($justificaRecusa == null) {
                            
                            $justificaRecusa = $dados["DSJUSTIFICARECUSA"];
                        } 
                        else {
                            
                            $justificaRecusa = $justificaRecusa.'    /    '.$dados["DSJUSTIFICARECUSA"];
                        }
                    } 
                    else if ($dados["IDSITUACAO"] == 'P'){
                        
                        $countPendente = $countPendente+1;
                    } 
                } 

                if ($countRecusado > 0) {
                    
                    $params = array(
                        "IDSITUACAO"        => 'R',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALORCA"  => $signatario,
                        "NRSEQORCAMENTO"    => $seqPropCont
                    );
                    
                    $this->assinaturaFactory->updateAssinaProposta($params);
                }
                else if ($countPendente > 0) {
                    
                    /* O registro permanece pendente */
                }
                else {
                    
                     $params = array(
                        "IDSITUACAO"        => 'A',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALORCA"  => $signatario,
                        "NRSEQORCAMENTO"    => $seqPropCont
                    );
                    
                    $this->assinaturaFactory->updateAssinaProposta($params);
                }
            }
            else {
                
                $params = array(
                    "NRASSDIGITALCONTR" => $signatario,
                    "NRSEQCONTRATO" => $seqPropCont
                );
                
                $dadosProposta = $this->assinaturaFactory->recebeDadosAssDocContr($params);  
                
                foreach ($dadosProposta as $dados){
                    
                    if ($dados["IDSITUACAO"] == 'A'){
                        
                        $countAprovado = $countAprovado+1;
                    } 
                    else if ($dados["IDSITUACAO"] == 'R'){ 
                        
                        $countRecusado = $countRecusado+1;
                        
                        if ($justificaRecusa == null) {
                            
                            $justificaRecusa = $dados["DSJUSTIFICARECUSA"];
                        } 
                        else {
                            
                            $justificaRecusa = $justificaRecusa.'<br><br>'.$dados["DSJUSTIFICARECUSA"];
                        }
                    } 
                    else if ($dados["IDSITUACAO"] == 'P'){
                        
                        $countPendente = $countPendente+1;
                    } 
                } 
                
                if ($countRecusado > 0) {
                    
                    $params = array(
                        "IDSITUACAO"        => 'R',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALCONTR"  => $signatario,
                        "NRSEQCONTRATO"    => $seqPropCont
                        
                    );
                    
                    $this->assinaturaFactory->updateAssinaContrato($params);
                }
                else if ($countPendente > 0) {
                    
                    /* O registro permanece pendente */
                }
                else {
                    
                     $params = array(
                        "IDSITUACAO"        => 'A',
                        "DSJUSTIFICARECUSA" => $justificaRecusa,
                        "NRASSDIGITALCONTR"  => $signatario,
                        "NRSEQCONTRATO"    => $seqPropCont
                    );
                    
                    $this->assinaturaFactory->updateAssinaContrato($params);
                }
            }
            
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function incluirSignatario($documentId, $signers, $eletrSigners) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantAdd', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
    
            $request->setHeader($headers);
    
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $body = $this->buildJsontoIncSignatario($documentId, $signers, $eletrSigners);
            
            $return['request'] = $body;
             
            // Request body
            $request->setBody($body);
        
            try {
                $response = $request->send();
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }

        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buildJsontoIncSignatario($documentId, $signers, $eletrSigners){

        /* -- Signatários Digitais -- */
        
        $signers_aux = array();
        
        foreach ($signers as $ind => $signer) {
        
            $signers_aux[$ind] = array(
                "step"  => $signer['POSICAO'],
                "title" => $signer['TIPO'],
                "name"  => $signer['NOME'],
                "email" => $signer['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $signer['CPF']
            );
        }
        
        /* -- Signatários Eletrônicos -- */
        
        $eletrSigners_aux = array();
        
        foreach ($eletrSigners as $ind => $eletr) {
        
            $eletrSigners_aux[$ind] = array(
                "step"  => $eletr['POSICAO'],
                "title" => $eletr['TIPO'],
                "name"  => $eletr['NOME'],
                "email" => $eletr['EMAIL'],
                //"email" => 'ranzobh@yahoo.com.br',
                "individualIdentificationCode"=> $eletr['CPF']
            );
        }
        
        /* -- Cria e envia o body do documento -- */

        $arrayCreate = array(
            "documentId" => $documentId,
            "signers" => $signers_aux,
            "electronicSigners" => $eletrSigners_aux
        );

        $jsonCreate = json_encode($arrayCreate, JSON_PRETTY_PRINT);

        return $jsonCreate;
    }
    
    public function excluirSignatario($documentId, $idsignatcertsign, $nrcpfsignatario) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/participantDiscard', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documentId" => $documentId,
                "individualIdentificationCode" => $nrcpfsignatario
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request'] = $body;
             
            try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function reenviarSolicitacao ($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/SendReminder', \Http_Request2::METHOD_POST);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Content-Type' => 'application/json',
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_POST);
            
            $bodyCreate = array(
                "documents" => [$documentId]
            );

            $body = json_encode($bodyCreate, JSON_PRETTY_PRINT);
            
            // Request body
            $request->setBody($body);
            
            $return['request'] = $body;
            
            try {
                $response = $request->send();

                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function excluirdocumento($documentId) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/delete/'.$documentId, \Http_Request2::METHOD_DELETE);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_DELETE);
            $body = null;
            // Request body
            $request->setBody($body);
            
             try {
                $response = $request->send();
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function dowloadDocumento($chave) {
        
        $token = $this->getTokenCertsign();
        
        $return = array();

        try {
            $request = new \Http_Request2($this->urlIntegracaoCertisign.'document/package?key='.$chave, \Http_Request2::METHOD_GET);
            $url = $request->getUrl();
    
            $headers = array(
                // Request headers
                'Token' => $token,
            );
            
            $request->setHeader($headers);
            
            $parameters = array(
                // Request parameters
                'key' => $chave,
                'includeOriginal' => true,
                'includeManifest' => true,
                'zipped' => true,
            );
            
            $url->setQueryVariables($parameters);
            
            $request->setMethod(\Http_Request2::METHOD_GET);
            $body = null;
            // Request body
            $request->setBody($body);
            
             try {
                $response = $request->send();
                
                
                $retorno =  $response->getBody();
                $retorno =  json_decode($retorno, true);
                
                $return['response'] = $retorno;
                
                return $return;
            }
            catch (\HttpException $ex)
            {
                $return['response'] = $ex;
    
                return $return;
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getTokenCertsign() {
        
        try {
           
           $nrorg = $this->environment->getNrOrg();
               
           $params = array(
                "NRORG" => $nrorg
           );
            
           $token = $this->assinaturaFactory->getTokenCertsign($params);
        
           $token = $token[0]["DSTOKENCERTSIGN"];   
           
           return $token;    
        
        } catch(\Exception $e) {
            throw $e;
        }   
    }
    
    
}